# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class Database(pulumi.CustomResource):
    allow_connections: pulumi.Output[bool]
    """
    If `false` then no one can connect to this
    database. The default is `true`, allowing connections (except as restricted by
    other mechanisms, such as `GRANT` or `REVOKE CONNECT`).
    """
    connection_limit: pulumi.Output[float]
    """
    How many concurrent connections can be
    established to this database. `-1` (the default) means no limit.
    """
    encoding: pulumi.Output[str]
    """
    Character set encoding to use in the new database
    """
    is_template: pulumi.Output[bool]
    """
    If `true`, then this database can be cloned by any
    user with `CREATEDB` privileges; if `false` (the default), then only
    superusers or the owner of the database can clone it.
    """
    lc_collate: pulumi.Output[str]
    """
    Collation order (LC_COLLATE) to use in the new database
    """
    lc_ctype: pulumi.Output[str]
    """
    Character classification (LC_CTYPE) to use in the new database
    """
    name: pulumi.Output[str]
    """
    The name of the database. Must be unique on the PostgreSQL
    server instance where it is configured.
    """
    owner: pulumi.Output[str]
    """
    The role name of the user who will own the database, or
    `DEFAULT` to use the default (namely, the user executing the command). To
    create a database owned by another role or to change the owner of an existing
    database, you must be a direct or indirect member of the specified role, or
    the username in the provider is a superuser.
    """
    tablespace_name: pulumi.Output[str]
    """
    The name of the tablespace that will be
    associated with the database, or `DEFAULT` to use the template database's
    tablespace.  This tablespace will be the default tablespace used for objects
    created in this database.
    """
    template: pulumi.Output[str]
    """
    The name of the template from which to create the new database
    """
    def __init__(__self__, resource_name, opts=None, allow_connections=None, connection_limit=None, encoding=None, is_template=None, lc_collate=None, lc_ctype=None, name=None, owner=None, tablespace_name=None, template=None, __props__=None, __name__=None, __opts__=None):
        """
        The ``.Database`` resource creates and manages [database
        objects](https://www.postgresql.org/docs/current/static/managing-databases.html)
        within a PostgreSQL server instance.


        ## Usage

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        my_db = postgresql.Database("myDb",
            allow_connections=True,
            connection_limit=-1,
            lc_collate="C",
            owner="my_role",
            template="template0")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] allow_connections: If `false` then no one can connect to this
               database. The default is `true`, allowing connections (except as restricted by
               other mechanisms, such as `GRANT` or `REVOKE CONNECT`).
        :param pulumi.Input[float] connection_limit: How many concurrent connections can be
               established to this database. `-1` (the default) means no limit.
        :param pulumi.Input[str] encoding: Character set encoding to use in the new database
        :param pulumi.Input[bool] is_template: If `true`, then this database can be cloned by any
               user with `CREATEDB` privileges; if `false` (the default), then only
               superusers or the owner of the database can clone it.
        :param pulumi.Input[str] lc_collate: Collation order (LC_COLLATE) to use in the new database
        :param pulumi.Input[str] lc_ctype: Character classification (LC_CTYPE) to use in the new database
        :param pulumi.Input[str] name: The name of the database. Must be unique on the PostgreSQL
               server instance where it is configured.
        :param pulumi.Input[str] owner: The role name of the user who will own the database, or
               `DEFAULT` to use the default (namely, the user executing the command). To
               create a database owned by another role or to change the owner of an existing
               database, you must be a direct or indirect member of the specified role, or
               the username in the provider is a superuser.
        :param pulumi.Input[str] tablespace_name: The name of the tablespace that will be
               associated with the database, or `DEFAULT` to use the template database's
               tablespace.  This tablespace will be the default tablespace used for objects
               created in this database.
        :param pulumi.Input[str] template: The name of the template from which to create the new database
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['allow_connections'] = allow_connections
            __props__['connection_limit'] = connection_limit
            __props__['encoding'] = encoding
            __props__['is_template'] = is_template
            __props__['lc_collate'] = lc_collate
            __props__['lc_ctype'] = lc_ctype
            __props__['name'] = name
            __props__['owner'] = owner
            __props__['tablespace_name'] = tablespace_name
            __props__['template'] = template
        super(Database, __self__).__init__(
            'postgresql:index/database:Database',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, allow_connections=None, connection_limit=None, encoding=None, is_template=None, lc_collate=None, lc_ctype=None, name=None, owner=None, tablespace_name=None, template=None):
        """
        Get an existing Database resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] allow_connections: If `false` then no one can connect to this
               database. The default is `true`, allowing connections (except as restricted by
               other mechanisms, such as `GRANT` or `REVOKE CONNECT`).
        :param pulumi.Input[float] connection_limit: How many concurrent connections can be
               established to this database. `-1` (the default) means no limit.
        :param pulumi.Input[str] encoding: Character set encoding to use in the new database
        :param pulumi.Input[bool] is_template: If `true`, then this database can be cloned by any
               user with `CREATEDB` privileges; if `false` (the default), then only
               superusers or the owner of the database can clone it.
        :param pulumi.Input[str] lc_collate: Collation order (LC_COLLATE) to use in the new database
        :param pulumi.Input[str] lc_ctype: Character classification (LC_CTYPE) to use in the new database
        :param pulumi.Input[str] name: The name of the database. Must be unique on the PostgreSQL
               server instance where it is configured.
        :param pulumi.Input[str] owner: The role name of the user who will own the database, or
               `DEFAULT` to use the default (namely, the user executing the command). To
               create a database owned by another role or to change the owner of an existing
               database, you must be a direct or indirect member of the specified role, or
               the username in the provider is a superuser.
        :param pulumi.Input[str] tablespace_name: The name of the tablespace that will be
               associated with the database, or `DEFAULT` to use the template database's
               tablespace.  This tablespace will be the default tablespace used for objects
               created in this database.
        :param pulumi.Input[str] template: The name of the template from which to create the new database
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["allow_connections"] = allow_connections
        __props__["connection_limit"] = connection_limit
        __props__["encoding"] = encoding
        __props__["is_template"] = is_template
        __props__["lc_collate"] = lc_collate
        __props__["lc_ctype"] = lc_ctype
        __props__["name"] = name
        __props__["owner"] = owner
        __props__["tablespace_name"] = tablespace_name
        __props__["template"] = template
        return Database(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

_SNAKE_TO_CAMEL_CASE_TABLE = {
    "allow_connections": "allowConnections",
    "bypass_row_level_security": "bypassRowLevelSecurity",
    "connection_limit": "connectionLimit",
    "create_database": "createDatabase",
    "create_role": "createRole",
    "drop_cascade": "dropCascade",
    "encrypted_password": "encryptedPassword",
    "if_not_exists": "ifNotExists",
    "is_template": "isTemplate",
    "lc_collate": "lcCollate",
    "lc_ctype": "lcCtype",
    "object_type": "objectType",
    "search_paths": "searchPaths",
    "skip_drop_role": "skipDropRole",
    "skip_reassign_owned": "skipReassignOwned",
    "statement_timeout": "statementTimeout",
    "tablespace_name": "tablespaceName",
    "valid_until": "validUntil",
    "with_grant_option": "withGrantOption",
}

_CAMEL_TO_SNAKE_CASE_TABLE = {
    "allowConnections": "allow_connections",
    "bypassRowLevelSecurity": "bypass_row_level_security",
    "connectionLimit": "connection_limit",
    "createDatabase": "create_database",
    "createRole": "create_role",
    "dropCascade": "drop_cascade",
    "encryptedPassword": "encrypted_password",
    "ifNotExists": "if_not_exists",
    "isTemplate": "is_template",
    "lcCollate": "lc_collate",
    "lcCtype": "lc_ctype",
    "objectType": "object_type",
    "searchPaths": "search_paths",
    "skipDropRole": "skip_drop_role",
    "skipReassignOwned": "skip_reassign_owned",
    "statementTimeout": "statement_timeout",
    "tablespaceName": "tablespace_name",
    "validUntil": "valid_until",
    "withGrantOption": "with_grant_option",
}



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import importlib
# Make subpackages available:
__all__ = ['config']
for pkg in __all__:
    if pkg != 'config':
        importlib.import_module(f'{__name__}.{pkg}')

# Export this package's modules as members:
from .database import *
from .default_privileg import *
from .default_privileges import *
from .extension import *
from .grant import *
from .provider import *
from .role import *
from .schema import *



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class Provider(pulumi.ProviderResource):
    def __init__(__self__, resource_name, opts=None, clientcert=None, connect_timeout=None, database=None, database_username=None, expected_version=None, host=None, max_connections=None, password=None, port=None, ssl_mode=None, sslmode=None, sslrootcert=None, superuser=None, username=None, __props__=None, __name__=None, __opts__=None):
        """
        The provider type for the postgresql package. By default, resources use package-wide configuration
        settings, however an explicit `Provider` instance may be created and passed during resource
        construction to achieve fine-grained programmatic control over provider settings. See the
        [documentation](https://www.pulumi.com/docs/reference/programming-model/#providers) for more information.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] clientcert: SSL client certificate if required by the database.
        :param pulumi.Input[float] connect_timeout: Maximum wait for connection, in seconds. Zero or not specified means wait indefinitely.
        :param pulumi.Input[str] database: The name of the database to connect to in order to conenct to (defaults to `postgres`).
        :param pulumi.Input[str] database_username: Database username associated to the connected user (for user name maps)
        :param pulumi.Input[str] expected_version: Specify the expected version of PostgreSQL.
        :param pulumi.Input[str] host: Name of PostgreSQL server address to connect to
        :param pulumi.Input[float] max_connections: Maximum number of connections to establish to the database. Zero means unlimited.
        :param pulumi.Input[str] password: Password to be used if the PostgreSQL server demands password authentication
        :param pulumi.Input[float] port: The PostgreSQL port number to connect to at the server host, or socket file name extension for Unix-domain connections
        :param pulumi.Input[str] sslmode: This option determines whether or with what priority a secure SSL TCP/IP connection will be negotiated with the
               PostgreSQL server
        :param pulumi.Input[str] sslrootcert: The SSL server root certificate file path. The file must contain PEM encoded data.
        :param pulumi.Input[bool] superuser: Specify if the user to connect as is a Postgres superuser or not.If not, some feature might be disabled (e.g.:
               Refreshing state password from Postgres)
        :param pulumi.Input[str] username: PostgreSQL user name to connect as

        The **clientcert** object supports the following:

          * `cert` (`pulumi.Input[str]`)
          * `key` (`pulumi.Input[str]`)
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['clientcert'] = pulumi.Output.from_input(clientcert).apply(json.dumps) if clientcert is not None else None
            if connect_timeout is None:
                connect_timeout = (utilities.get_env_int('PGCONNECT_TIMEOUT') or 180)
            __props__['connect_timeout'] = pulumi.Output.from_input(connect_timeout).apply(json.dumps) if connect_timeout is not None else None
            if database is None:
                database = (utilities.get_env('PGDATABASE') or 'postgres')
            __props__['database'] = database
            __props__['database_username'] = database_username
            __props__['expected_version'] = expected_version
            if host is None:
                host = utilities.get_env('PGHOST')
            __props__['host'] = host
            __props__['max_connections'] = pulumi.Output.from_input(max_connections).apply(json.dumps) if max_connections is not None else None
            if password is None:
                password = utilities.get_env('PGPASSWORD')
            __props__['password'] = password
            if port is None:
                port = (utilities.get_env_int('PGPORT') or 5432)
            __props__['port'] = pulumi.Output.from_input(port).apply(json.dumps) if port is not None else None
            __props__['ssl_mode'] = ssl_mode
            if sslmode is None:
                sslmode = utilities.get_env('PGSSLMODE')
            __props__['sslmode'] = sslmode
            __props__['sslrootcert'] = sslrootcert
            __props__['superuser'] = pulumi.Output.from_input(superuser).apply(json.dumps) if superuser is not None else None
            if username is None:
                username = (utilities.get_env('PGUSER') or 'postgres')
            __props__['username'] = username
        super(Provider, __self__).__init__(
            'postgresql',
            resource_name,
            __props__,
            opts)

    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class Schema(pulumi.CustomResource):
    database: pulumi.Output[str]
    """
    The DATABASE in which where this schema will be created. (Default: The database used by your `provider` configuration)
    """
    drop_cascade: pulumi.Output[bool]
    """
    When true, will also drop all the objects that are contained in the schema. (Default: false)
    """
    if_not_exists: pulumi.Output[bool]
    """
    When true, use the existing schema if it exists. (Default: true)
    """
    name: pulumi.Output[str]
    """
    The name of the schema. Must be unique in the PostgreSQL
    database instance where it is configured.
    """
    owner: pulumi.Output[str]
    """
    The ROLE who owns the schema.
    """
    policies: pulumi.Output[list]
    """
    Can be specified multiple times for each policy.  Each
    policy block supports fields documented below.

      * `create` (`bool`) - Should the specified ROLE have CREATE privileges to the specified SCHEMA.
      * `createWithGrant` (`bool`) - Should the specified ROLE have CREATE privileges to the specified SCHEMA and the ability to GRANT the CREATE privilege to other ROLEs.
      * `role` (`str`) - The ROLE who is receiving the policy.  If this value is empty or not specified it implies the policy is referring to the [`PUBLIC` role](https://www.postgresql.org/docs/current/static/sql-grant.html).
      * `usage` (`bool`) - Should the specified ROLE have USAGE privileges to the specified SCHEMA.
      * `usageWithGrant` (`bool`) - Should the specified ROLE have USAGE privileges to the specified SCHEMA and the ability to GRANT the USAGE privilege to other ROLEs.
    """
    def __init__(__self__, resource_name, opts=None, database=None, drop_cascade=None, if_not_exists=None, name=None, owner=None, policies=None, __props__=None, __name__=None, __opts__=None):
        """
        The ``.Schema`` resource creates and manages [schema
        objects](https://www.postgresql.org/docs/current/static/ddl-schemas.html) within
        a PostgreSQL database.


        ## Usage

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        app_www = postgresql.Role("appWww")
        app_dba = postgresql.Role("appDba")
        app_releng = postgresql.Role("appReleng")
        my_schema = postgresql.Schema("mySchema",
            owner="postgres",
            policies=[
                {
                    "role": app_www.name,
                    "usage": True,
                },
                {
                    "create": True,
                    "role": app_releng.name,
                    "usage": True,
                },
                {
                    "createWithGrant": True,
                    "role": app_dba.name,
                    "usageWithGrant": True,
                },
            ])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] database: The DATABASE in which where this schema will be created. (Default: The database used by your `provider` configuration)
        :param pulumi.Input[bool] drop_cascade: When true, will also drop all the objects that are contained in the schema. (Default: false)
        :param pulumi.Input[bool] if_not_exists: When true, use the existing schema if it exists. (Default: true)
        :param pulumi.Input[str] name: The name of the schema. Must be unique in the PostgreSQL
               database instance where it is configured.
        :param pulumi.Input[str] owner: The ROLE who owns the schema.
        :param pulumi.Input[list] policies: Can be specified multiple times for each policy.  Each
               policy block supports fields documented below.

        The **policies** object supports the following:

          * `create` (`pulumi.Input[bool]`) - Should the specified ROLE have CREATE privileges to the specified SCHEMA.
          * `createWithGrant` (`pulumi.Input[bool]`) - Should the specified ROLE have CREATE privileges to the specified SCHEMA and the ability to GRANT the CREATE privilege to other ROLEs.
          * `role` (`pulumi.Input[str]`) - The ROLE who is receiving the policy.  If this value is empty or not specified it implies the policy is referring to the [`PUBLIC` role](https://www.postgresql.org/docs/current/static/sql-grant.html).
          * `usage` (`pulumi.Input[bool]`) - Should the specified ROLE have USAGE privileges to the specified SCHEMA.
          * `usageWithGrant` (`pulumi.Input[bool]`) - Should the specified ROLE have USAGE privileges to the specified SCHEMA and the ability to GRANT the USAGE privilege to other ROLEs.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['database'] = database
            __props__['drop_cascade'] = drop_cascade
            __props__['if_not_exists'] = if_not_exists
            __props__['name'] = name
            __props__['owner'] = owner
            __props__['policies'] = policies
        super(Schema, __self__).__init__(
            'postgresql:index/schema:Schema',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, database=None, drop_cascade=None, if_not_exists=None, name=None, owner=None, policies=None):
        """
        Get an existing Schema resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] database: The DATABASE in which where this schema will be created. (Default: The database used by your `provider` configuration)
        :param pulumi.Input[bool] drop_cascade: When true, will also drop all the objects that are contained in the schema. (Default: false)
        :param pulumi.Input[bool] if_not_exists: When true, use the existing schema if it exists. (Default: true)
        :param pulumi.Input[str] name: The name of the schema. Must be unique in the PostgreSQL
               database instance where it is configured.
        :param pulumi.Input[str] owner: The ROLE who owns the schema.
        :param pulumi.Input[list] policies: Can be specified multiple times for each policy.  Each
               policy block supports fields documented below.

        The **policies** object supports the following:

          * `create` (`pulumi.Input[bool]`) - Should the specified ROLE have CREATE privileges to the specified SCHEMA.
          * `createWithGrant` (`pulumi.Input[bool]`) - Should the specified ROLE have CREATE privileges to the specified SCHEMA and the ability to GRANT the CREATE privilege to other ROLEs.
          * `role` (`pulumi.Input[str]`) - The ROLE who is receiving the policy.  If this value is empty or not specified it implies the policy is referring to the [`PUBLIC` role](https://www.postgresql.org/docs/current/static/sql-grant.html).
          * `usage` (`pulumi.Input[bool]`) - Should the specified ROLE have USAGE privileges to the specified SCHEMA.
          * `usageWithGrant` (`pulumi.Input[bool]`) - Should the specified ROLE have USAGE privileges to the specified SCHEMA and the ability to GRANT the USAGE privilege to other ROLEs.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["database"] = database
        __props__["drop_cascade"] = drop_cascade
        __props__["if_not_exists"] = if_not_exists
        __props__["name"] = name
        __props__["owner"] = owner
        __props__["policies"] = policies
        return Schema(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from . import utilities, tables

class Grant(pulumi.CustomResource):
    database: pulumi.Output[str]
    """
    The database to grant privileges on for this role.
    """
    object_type: pulumi.Output[str]
    """
    The PostgreSQL object type to grant the privileges on (one of: table, sequence).
    """
    privileges: pulumi.Output[list]
    """
    The list of privileges to grant.
    """
    role: pulumi.Output[str]
    """
    The name of the role to grant privileges on.
    """
    schema: pulumi.Output[str]
    """
    The database schema to grant privileges on for this role.
    """
    with_grant_option: pulumi.Output[bool]
    """
    Permit the grant recipient to grant it to others
    """
    def __init__(__self__, resource_name, opts=None, database=None, object_type=None, privileges=None, role=None, schema=None, with_grant_option=None, __props__=None, __name__=None, __opts__=None):
        """
        The ``.Grant`` resource creates and manages privileges given to a user for a database schema.

        > **Note:** This resource needs Postgresql version 9 or above.

        ## Usage

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        readonly_tables = postgresql.Grant("readonlyTables",
            database="test_db",
            object_type="table",
            privileges=["SELECT"],
            role="test_role",
            schema="public")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] database: The database to grant privileges on for this role.
        :param pulumi.Input[str] object_type: The PostgreSQL object type to grant the privileges on (one of: table, sequence).
        :param pulumi.Input[list] privileges: The list of privileges to grant.
        :param pulumi.Input[str] role: The name of the role to grant privileges on.
        :param pulumi.Input[str] schema: The database schema to grant privileges on for this role.
        :param pulumi.Input[bool] with_grant_option: Permit the grant recipient to grant it to others
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if database is None:
                raise TypeError("Missing required property 'database'")
            __props__['database'] = database
            if object_type is None:
                raise TypeError("Missing required property 'object_type'")
            __props__['object_type'] = object_type
            if privileges is None:
                raise TypeError("Missing required property 'privileges'")
            __props__['privileges'] = privileges
            if role is None:
                raise TypeError("Missing required property 'role'")
            __props__['role'] = role
            __props__['schema'] = schema
            __props__['with_grant_option'] = with_grant_option
        super(Grant, __self__).__init__(
            'postgresql:index/grant:Grant',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, database=None, object_type=None, privileges=None, role=None, schema=None, with_grant_option=None):
        """
        Get an existing Grant resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] database: The database to grant privileges on for this role.
        :param pulumi.Input[str] object_type: The PostgreSQL object type to grant the privileges on (one of: table, sequence).
        :param pulumi.Input[list] privileges: The list of privileges to grant.
        :param pulumi.Input[str] role: The name of the role to grant privileges on.
        :param pulumi.Input[str] schema: The database schema to grant privileges on for this role.
        :param pulumi.Input[bool] with_grant_option: Permit the grant recipient to grant it to others
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["database"] = database
        __props__["object_type"] = object_type
        __props__["privileges"] = privileges
        __props__["role"] = role
        __props__["schema"] = schema
        __props__["with_grant_option"] = with_grant_option
        return Grant(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

__config__ = pulumi.Config('postgresql')

clientcert = __config__.get('clientcert')
"""
SSL client certificate if required by the database.
"""

connect_timeout = __config__.get('connectTimeout') or (utilities.get_env_int('PGCONNECT_TIMEOUT') or 180)
"""
Maximum wait for connection, in seconds. Zero or not specified means wait indefinitely.
"""

database = __config__.get('database') or (utilities.get_env('PGDATABASE') or 'postgres')
"""
The name of the database to connect to in order to conenct to (defaults to `postgres`).
"""

database_username = __config__.get('databaseUsername')
"""
Database username associated to the connected user (for user name maps)
"""

expected_version = __config__.get('expectedVersion')
"""
Specify the expected version of PostgreSQL.
"""

host = __config__.get('host') or utilities.get_env('PGHOST')
"""
Name of PostgreSQL server address to connect to
"""

max_connections = __config__.get('maxConnections')
"""
Maximum number of connections to establish to the database. Zero means unlimited.
"""

password = __config__.get('password') or utilities.get_env('PGPASSWORD')
"""
Password to be used if the PostgreSQL server demands password authentication
"""

port = __config__.get('port') or (utilities.get_env_int('PGPORT') or 5432)
"""
The PostgreSQL port number to connect to at the server host, or socket file name extension for Unix-domain connections
"""

ssl_mode = __config__.get('sslMode')

sslmode = __config__.get('sslmode') or utilities.get_env('PGSSLMODE')
"""
This option determines whether or with what priority a secure SSL TCP/IP connection will be negotiated with the
PostgreSQL server
"""

sslrootcert = __config__.get('sslrootcert')
"""
The SSL server root certificate file path. The file must contain PEM encoded data.
"""

superuser = __config__.get('superuser')
"""
Specify if the user to connect as is a Postgres superuser or not.If not, some feature might be disabled (e.g.:
Refreshing state password from Postgres)
"""

username = __config__.get('username') or (utilities.get_env('PGUSER') or 'postgres')
"""
PostgreSQL user name to connect as
"""

